@@ -17,6 +17,7 @@
 namespace infomap {
 
 class InfoNode;
+class StateNetwork;
 
 class BiasedMapEquation : protected MapEquation {
   using Base = MapEquation;
@@ -31,7 +32,11 @@ public:
       : MapEquation(other),
         preferredNumModules(other.preferredNumModules),
         currentNumModules(other.currentNumModules),
-        biasedCost(other.biasedCost)
+        biasedCost(other.biasedCost),
+        useEntropyBiasCorrection(other.useEntropyBiasCorrection),
+        indexEntropyBiasCorrection(other.indexEntropyBiasCorrection),
+        moduleEntropyBiasCorrection(other.moduleEntropyBiasCorrection),
+        gamma(other.gamma)
   {
   }
 
@@ -41,6 +46,10 @@ public:
     preferredNumModules = other.preferredNumModules;
     currentNumModules = other.currentNumModules;
     biasedCost = other.biasedCost;
+    useEntropyBiasCorrection = other.useEntropyBiasCorrection;
+    indexEntropyBiasCorrection = other.indexEntropyBiasCorrection;
+    moduleEntropyBiasCorrection = other.moduleEntropyBiasCorrection;
+    gamma = other.gamma;
     return *this;
   }
 
@@ -50,7 +59,8 @@ public:
   // Getters
   // ===================================================
 
-  using Base::getIndexCodelength;
+  // using Base::getIndexCodelength;
+  double getIndexCodelength() const;
 
   // double getModuleCodelength() const { return moduleCodelength + metaCodelength; };
   double getModuleCodelength() const;
@@ -58,6 +68,8 @@ public:
   // double getCodelength() const { return codelength + metaCodelength; };
   double getCodelength() const;
 
+  double getEntropyBiasCorrection() const;
+
   // ===================================================
   // IO
   // ===================================================
@@ -119,6 +131,7 @@ protected:
   // Protected member functions
   // ===================================================
   double calcCodelengthOnModuleOfLeafNodes(const InfoNode& parent) const;
+  double calcCodelengthOnModuleOfModules(const InfoNode& parent) const;
 
   int getDeltaNumModulesIfMoving(InfoNode& current,
                                  unsigned int oldModule,
@@ -145,6 +158,10 @@ protected:
 
   double calcNumModuleCost(unsigned int numModules) const;
 
+  double calcIndexEntropyBiasCorrection(unsigned int numModules) const;
+  double calcModuleEntropyBiasCorrection(unsigned int numModules) const;
+  double calcEntropyBiasCorrection(unsigned int numModules) const;
+
   // ===================================================
   // Consolidation
   // ===================================================
@@ -165,6 +182,8 @@ protected:
   // Protected member functions
   // ===================================================
 
+  double correctionCoefficient() const { return gamma * entropyBiasCorrectionMultiplier; }
+
   /**
    *  Get meta codelength of module of current node
    * @param addRemoveOrNothing +1, -1 or 0 to calculate codelength
@@ -191,6 +210,18 @@ protected:
   unsigned int preferredNumModules = 0;
   unsigned int currentNumModules = 0;
   double biasedCost = 0.0;
+
+  // For entropy bias correction
+  bool useEntropyBiasCorrection = false;
+  double entropyBiasCorrectionMultiplier = 1;
+  double indexEntropyBiasCorrection = 0;
+  double moduleEntropyBiasCorrection = 0;
+  double gamma = 0.7;
+  static double s_totalDegree;
+  static unsigned int s_numNodes;
+
+  public:
+  static void setNetworkProperties(const StateNetwork& network);
 };
 
 
